package no.ntnu.idatt1002.demo.data.recipes;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

// Method in case we want to check if also the amount is enough for the recipe.
// Recipes without any ingredients can be showed among all ingredients, but not in suggestion based on ingredients
// at hand.
//TODO: Record?
public class Recipe {

    private String name = "";
    private List<RecipeIngredient> ingredientList = new ArrayList<>();
    private String instructions = "";

    public Recipe(String name, String description ) {
        if(name.isBlank() | description.isBlank()) {
            throw new IllegalArgumentException("The recipe must have a name and a description.");
        }
        this.name = name;
        this.instructions = description;
    }


    public String getName() {
        return name;
    }

    public void setName(String name) {
        if(name.isBlank()) {
            throw new IllegalArgumentException("The recipe name cannot be left blank.");
        }
        this.name = name;
    }

    public List<RecipeIngredient> getIngredientList() {
        return ingredientList;
    }

    public void setIngredientList(List<RecipeIngredient> ingredientList) {
        this.ingredientList = ingredientList;
    }

    public String getInstructions() {
        return instructions;
    }

    public void setInstructions(String instructions) {
        this.instructions = instructions;
    }

    public void addIngredient(RecipeIngredient ingredient) {
        // Remove old ingredient if it is the same type as added. Replace.
        this.ingredientList.add(ingredient);
    }

    public void addIngredients(ArrayList<RecipeIngredient> ingredients) {
        ingredients.forEach((ingredient) -> this.ingredientList.add(ingredient));
    }

    /**
     * The functionality may be expanded upon in order to also check if the amount is sufficient for the current recipe.
     * @param ingredientsAtHand
     */
    public void updateIngredientStatus(IngredientsAtHand ingredientsAtHand) {
        // Will need a supporting class for converting between units to be accurate.
        ingredientList.stream().forEach((inRecipe) -> {
            ingredientsAtHand.getIngredientsAtHand().stream().forEach((atHand) -> {
                if(inRecipe.getFoodType() == atHand.getFoodType()) {
                    inRecipe.setAtHand(true);
                }
            });
        });
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Recipe recipe = (Recipe) o;
        return Objects.equals(name, recipe.name);
    }

    @Override
    public int hashCode() {
        return Objects.hash(name, ingredientList, instructions);
    }
}
